/*
 * $Id: $
 *
 */

/* GNUPLOT - doc2sgml.c */

/*[
 * Copyright 1999   Lars Hecking
 *
 * Permission to use, copy, and distribute this software and its
 * documentation for any purpose with or without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.
 *
 * Permission to modify the software is granted, but not the right to
 * distribute the complete modified source code.  Modifications are to
 * be distributed as patches to the released version.  Permission to
 * distribute binaries produced by compiling modified sources is granted,
 * provided you
 *   1. distribute the corresponding source modifications from the
 *    released version in the form of a patch file along with the binaries,
 *   2. add special version identification to distinguish your version
 *    in addition to the base release version number,
 *   3. provide your name and address as the primary contact for the
 *    support of your modified version, and
 *   4. retain our contact information in regard to use of the base
 *    software.
 * Permission to distribute the released version of the source code along
 * with corresponding source modifications in the form of a patch file is
 * granted with same provisions 2 through 4 for binary distributions.
 *
 * This software is provided "as is" without express or implied warranty
 * to the extent permitted by applicable law.
]*/

/*
 * doc2sgml.c  -- program to convert Gnuplot .DOC format to SGML document
 * Modified from doc2tex.c
 *
 *
 * usage:  doc2sgml [file.doc [file.tex]]
 *
 *   where file.doc is a Gnuplot .DOC file, and file.sgml will be an
 *     article document suitable for further processing with SGML tools.
 *
 * typical usage for GNUPLOT:
 *
 *   doc2sgml gnuplot.doc gnuplot.sgml
 *   sgml2html gnuplot.sgml
 *   sgml2info gnuplot.sgml
 *   sgml2latex gnuplot.sgml ; latex gnuplot.tex
 *   sgml2rtf gnuplot.sgml
 *   sgml2txt gnuplot.sgml
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "ansichek.h"
#include "stdfn.h"
#include "doc2x.h"

void init __PROTO((FILE *));
void convert __PROTO((FILE *, FILE *));
void process_line __PROTO((char *, FILE *));
void section __PROTO((char *, FILE *));
void putsgml __PROTO((char *, FILE *));
void puthtml __PROTO((char *, FILE *));
void finish __PROTO((FILE *));

static boolean intable = FALSE;
static boolean verb = FALSE;

int main(argc, argv)
int argc;
char **argv;
{
    FILE *infile;
    FILE *outfile;

    infile = stdin;
    outfile = stdout;

    if (argc > 3) {
	fprintf(stderr, "Usage: %s [infile [outfile]]\n", argv[0]);
	exit(EXIT_FAILURE);
    }
    if (argc >= 2) {
	if ((infile = fopen(argv[1], "r")) == (FILE *) NULL) {
	    fprintf(stderr, "%s: Can't open %s for reading\n",
		    argv[0], argv[1]);
	    exit(EXIT_FAILURE);
	}
    }
    if (argc == 3) {
	if ((outfile = fopen(argv[2], "w")) == (FILE *) NULL) {
	    fprintf(stderr, "%s: Can't open %s for writing\n",
		    argv[0], argv[2]);
	    exit(EXIT_FAILURE);
	}
    }
    convert(infile, outfile);
    finish(outfile);
    exit(EXIT_SUCCESS);
}


void convert(a, b)
FILE *a, *b;
{
    static char line[MAX_LINE_LEN + 1];

    /* write document preamble */

    fprintf(b, "<!doctype linuxdoc system>\n\n\
<!-- This file is gnuplot.sgml created by doc2sgml from gnuplot.doc.\n\
 -->\n\n\
<report>\n\n\
<!-- Title information -->\n\n\
<title>Gnuplot\n\
An Interactive Plotting Program\n\
<author>Thomas Williams, Colin Kelley\n\
<date>3 December 1998\n\
<abstract>\n\
This manual was prepared by Dick Crawford\n\
</abstract>\n\n\
<!-- Table of contents -->\n\
<toc>\n\n\
<!-- Begin the document -->\n\n");

    /* process each line of the file */
    while (get_line(line, sizeof(line), a))
	process_line(line, b);

}

void process_line(line, b)
char *line;
FILE *b;
{
    switch (line[0]) {		/* control character */
    case '?':{			/* interactive help entry */
	    break;		/* ignore */
	}
    case '@':{			/* start/end table */
	    if (intable) {
		(void) fputs("</verb></tscreen>\n", b);
		intable = FALSE;
	    } else {
		if (verb) {
		    (void) fputs("</verb></tscreen>\n", b);
		    verb = FALSE;
		}
		(void) fputs("<tscreen><verb>\n", b);
		intable = TRUE;
	    }
	    /* ignore rest of line */
	    break;
	}
    case '#':{			/* latex table entry */
	    break;		/* ignore */
	}
    case '^':{			/* external link escape */
	puthtml(line+1, b);
	break;
        }
    case '%':{			/* troff table entry */
	    break;		/* ignore */
	}
    case '\n':			/* empty text line */
    case ' ':{			/* normal text line */
	    if (intable) {
		if (line[1] == ' ') {
		    putsgml(line, b);
		}
		break;		/* ignore while in table */
	    }
	    if (line[1] == ' ') {
		/* verbatim mode */
		if (!verb) {
		    (void) fputs("<tscreen><verb>\n", b);
		    verb = TRUE;
		}
		(void) fputs(line + 2, b);
	    } else {
		if (verb) {
		    (void) fputs("</verb></tscreen>\n", b);
		    verb = FALSE;
		}
		if (line[0] == '\n')
		    putsgml(line, b);	/* handle totally blank line */
		else
		    putsgml(line + 1, b);
	    }
	    break;
	}
    default:{
	    if (isdigit((int) line[0])) {	/* start of section */
		if (!intable)	/* ignore while in table */
		    section(line, b);
	    } else
		fprintf(stderr, "unknown control code '%c' in column 1\n",
			line[0]);
	    break;
	}
    }
}

/* process a line with a digit control char */
/* starts a new [sub]section */

void section(line, b)
char *line;
FILE *b;
{
    static char string[MAX_LINE_LEN + 1];
    int sh_i;

    if (verb) {
	(void) fputs("</verb></tscreen>\n", b);
	verb = FALSE;
    }
    (void) sscanf(line, "%d %[^\n]s", &sh_i, string);
    switch (sh_i) {
    case 1:
	(void) fprintf(b, "<chapt>");
	break;
    case 2:
	(void) fprintf(b, "<sect>");
	break;
    case 3:
	(void) fprintf(b, "<sect1>");
	break;
    case 4:
	(void) fprintf(b, "<sect2>");
	break;
    case 5:
	(void) fprintf(b, "<sect3>");
	break;
    default:
    case 6:
	(void) fprintf(b, "\n<p>\n");
	break;
    }
    if (islower((int) string[0]))
	string[0] = toupper(string[0]);
    putsgml(string, b);
    (void) fprintf(b, "\n\n<p>\n");
}

/* put text in string str to file while buffering special SGML characters */
void putsgml(str, file)
FILE *file;
register char *str;
{
    register char ch;
    static boolean inquote = FALSE;
    int i;

    while ((ch = *str++) != NUL) {
	switch (ch) {
	case '#':
	    (void) fputs("&num;", file);
	    break;
	case '$':
	    (void) fputs("&dollar;", file);
	    break;
	case '%':
	    (void) fputs("&percnt;", file);
	    break;
	case '&':
	    if (intable)
		(void) fputs("&ero;", file);
	    else
		(void) fputs("&amp;", file);
	    break;
	case '{':
	    (void) fputs("&lcub;", file);
	    break;
	case '}':
	    (void) fputs("&rcub;", file);
	    break;
	case '[':
	    (void) fputs("&lsqb;", file);
	    break;
	case ']':
	    (void) fputs("&rsqb;", file);
	    break;
	case '\\':
	    (void) fputs("&bsol;", file);
	    break;
	case '~':
	    (void) fputs("&tilde;", file);
	    break;
	case '>':
	    (void) fputs("&gt;", file);
	    break;
	case '<':
	    (void) fputs("&lt;", file);
	    break;
	case '|':
	    (void) fputs("&verbar;", file);
	    break;
	case '"':
	    (void) fputs("&dquot;", file);
	    break;
	case '\'':
	    if (*str == '\'') {
		(void) fputs("&apos;", file);
		str++;
	    } else {
		(void) fputc(ch, file);
	    }
	    break;
	case '-':
	    if ((*str == '-') && (*(str + 1) == '-')) {
		(void) fputs(" &horbar; ", file);
		str += 2;
	    } else {
		(void) fputc(ch, file);
	    }
	    break;
	case '`':		/* backquotes mean boldface */
	    if (inquote) {
		(void) fputs("/", file);
		inquote = FALSE;
	    } else {
		(void) fputs("<bf/", file);
		inquote = TRUE;
	    }
	    break;
	case '_':		/* emphasised text ? */
	    for (i = 0; isalpha((int) (*(str + i))); i++);
	    if ((i > 0) && (*(str + i) == '_') &&
		isspace((int) (*(str + i + 1)))) {
		(void) fputs("<em/", file);
		for (; *str != '_'; str++) {
		    (void) fputc(*str, file);
		}
		str++;
		(void) fputs("/", file);
	    } else {
		(void) fputs("&lowbar;", file);
	    }
	    break;
	default:
	    (void) fputc(ch, file);
	    break;
	}
    }
}


/* Translate HTML tags to SGML tags in str and output to b */
void puthtml(str,b)
char *str;
FILE *b;
{
    int i = 0;
    return;
/*    (void) fputs(str, b); */
    while (str[i] != '\0') {
	if (str[i] != '<')
	    fputc(str[i], b);
	else {
	    while (str[i] != '>')
	    i++;
	}
	i++;
    }
}

void finish(b)
FILE *b;
{
    (void) fputs("</report>\n", b);
}
